#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import datetime
import mozinfo
import sys
import math
from argparse import ArgumentParser
from mozlog.structured import commandline, get_default_logger

from mozregression import errors
from mozregression import __version__
from mozregression.utils import get_date, date_of_release, format_date
from mozregression.runnightly import NightlyRunner, parse_bits
from mozregression.runinbound import InboundRunner
from mozregression.inboundfinder import get_repo_url


def compute_steps_left(steps):
    if steps <= 1:
        return 0
    return math.trunc(math.log(steps, 2))


class Bisector(object):

    curr_date = ''
    found_repo = None

    def __init__(self, nightly_runner, inbound_runner, appname="firefox",
                 last_good_revision=None, first_bad_revision=None):
        self.nightly_runner = nightly_runner
        self.inbound_runner = inbound_runner
        self.appname = appname
        self.last_good_revision = last_good_revision
        self.first_bad_revision = first_bad_revision
        self._logger = get_default_logger('Bisector')

    def find_regression_chset(self, last_good_revision, first_bad_revision):
        # Uses mozcommitbuilder to bisect on changesets
        # Only needed if they want to bisect, so we'll put the dependency here.
        from mozcommitbuilder import builder
        commit_builder = builder.Builder()

        self._logger.info(" Narrowed changeset range from %s to %s"
                          % (last_good_revision, first_bad_revision))

        self._logger.info("Time to do some bisecting and building!")
        commit_builder.bisect(last_good_revision, first_bad_revision)
        quit()

    def offer_build(self, last_good_revision, first_bad_revision):
        verdict = raw_input("do you want to bisect further by fetching"
                            " the repository and building? (y or n) ")
        if verdict != "y":
            sys.exit()

        if self.appname == "firefox":
            self.find_regression_chset(last_good_revision, first_bad_revision)
        else:
            sys.exit("Bisection on anything other than firefox is not"
                     " currently supported.")

    def print_range(self, good_date=None, bad_date=None):
        def _print_date_revision(name, revision, date):
            if date and revision:
                self._logger.info("%s revision: %s (%s)"
                                  % (name, revision, date))
            elif revision:
                self._logger.info("%s revision: %s" % (name, revision))
            elif date:
                self._logger.info("%s build: %s" % (name, date))
        _print_date_revision("Last good", self.last_good_revision, good_date)
        _print_date_revision("First bad", self.first_bad_revision, bad_date)

        self._logger.info("Pushlog:\n%s\n"
                          % self.get_pushlog_url(good_date, bad_date))

    def _ensure_metadata(self, good_date, bad_date):
        self._logger.info("Ensuring we have enough metadata to get a pushlog...")
        if not self.last_good_revision:
            self.found_repo, self.last_good_revision = \
                self.nightly_runner.get_build_info(good_date)

        if not self.first_bad_revision:
            self.found_repo, self.first_bad_revision = \
                self.nightly_runner.get_build_info(bad_date)

    def _get_verdict(self, build_type, offer_skip=True):
        verdict = ""
        options = ['good', 'g', 'bad', 'b', 'retry', 'r']
        if offer_skip:
            options += ['skip', 's']
        options += ['exit']
        while verdict not in options:
            verdict = raw_input("Was this %s build good, bad, or broken?"
                                " (type 'good', 'bad', 'skip', 'retry', or"
                                " 'exit' and press Enter): " % build_type)

        # shorten verdict to one character for processing...
        if len(verdict) > 1:
            return verdict[0]

        return verdict

    def print_inbound_regression_progress(self, revisions, revisions_left):
        self._logger.info("Narrowed inbound regression window from [%s, %s]"
                          " (%d revisions) to [%s, %s] (%d revisions)"
                          " (~%d steps left)"
                          % (revisions[0]['revision'],
                             revisions[-1]['revision'],
                             len(revisions),
                             revisions_left[0]['revision'],
                             revisions_left[-1]['revision'],
                             len(revisions_left),
                             compute_steps_left(len(revisions_left))))

    def bisect_inbound(self, inbound_revisions=None):
        self.found_repo = get_repo_url(
            inbound_branch=self.inbound_runner.inbound_branch)

        if not inbound_revisions:
            self._logger.info("Getting inbound builds between %s and %s"
                              % (self.last_good_revision,
                                 self.first_bad_revision))
            # anything within twelve hours is potentially within the range
            # (should be a tighter but some older builds have wrong timestamps,
            # see https://bugzilla.mozilla.org/show_bug.cgi?id=1018907 ...
            # we can change this at some point in the future, after those builds
            # expire)
            inbound_revisions = self.inbound_runner.app.build_finder \
                .get_build_infos(self.last_good_revision,
                                 self.first_bad_revision,
                                 range=60*60*12)

        mid = inbound_revisions.mid_point()
        if mid == 0:
            self._logger.info("Oh noes, no (more) inbound revisions :(")
            self.print_range()
            self.offer_build(self.last_good_revision,
                             self.first_bad_revision)
            return
        # hardcode repo to mozilla-central (if we use inbound, we may be
        # missing some revisions that went into the nightlies which we may
        # also be comparing against...)

        self._logger.info("Testing inbound build with timestamp %s,"
                          " revision %s"
                          % (inbound_revisions[mid]['timestamp'],
                             inbound_revisions[mid]['revision']))
        self.inbound_runner.start(inbound_revisions[mid]['timestamp'])

        verdict = self._get_verdict('inbound', offer_skip=False)
        self.inbound_runner.stop()
        info = self.inbound_runner.get_app_info()
        if verdict == 'g':
            self.last_good_revision = info['application_changeset']
        elif verdict == 'b':
            self.first_bad_revision = info['application_changeset']
        elif verdict == 'r':
            # do the same thing over again
            self.bisect_inbound(inbound_revisions=inbound_revisions)
            return
        elif verdict == 'e':
            self._logger.info('Newest known good inbound revision: %s'
                              % self.last_good_revision)
            self._logger.info('Oldest known bad inbound revision: %s'
                              % self.first_bad_revision)

            self._logger.info('To resume, run:')
            self.inbound_runner.print_resume_info(self.last_good_revision,
                                                  self.first_bad_revision)
            return

        if len(inbound_revisions) > 1 and verdict in ('g', 'b'):
            if verdict == 'g':
                revisions_left = inbound_revisions[(mid+1):]
            else:
                revisions_left = inbound_revisions[:mid]
            revisions_left.ensure_limits()
            self.print_inbound_regression_progress(inbound_revisions,
                                                   revisions_left)
            self.bisect_inbound(revisions_left)
        else:
            # no more inbounds to be bisect, we must build
            self._logger.info("No more inbounds to bisect")
            self.print_range()
            self.offer_build(self.last_good_revision, self.first_bad_revision)

    def print_nightly_regression_progress(self, good_date, bad_date,
                                          next_good_date, next_bad_date):
        next_days_range = (next_bad_date - next_good_date).days
        self._logger.info("Narrowed nightly regression window from"
                          " [%s, %s] (%d days) to [%s, %s] (%d days)"
                          " (~%d steps left)"
                          % (format_date(good_date),
                             format_date(bad_date),
                             (bad_date - good_date).days,
                             format_date(next_good_date),
                             format_date(next_bad_date),
                             next_days_range,
                             compute_steps_left(next_days_range)))

    def bisect_nightlies(self, good_date, bad_date, skips=0):
        mid_date = good_date + (bad_date - good_date) / 2

        mid_date += datetime.timedelta(days=skips)

        if mid_date == bad_date or mid_date == good_date:
            self._logger.info("Got as far as we can go bisecting nightlies...")
            self._ensure_metadata(good_date, bad_date)
            self.print_range(good_date, bad_date)
            if self.appname in ('firefox', 'fennec', 'b2g'):
                self._logger.info("... attempting to bisect inbound builds"
                                  " (starting from previous week, to make"
                                  " sure no inbound revision is missed)")
                prev_date = good_date - datetime.timedelta(days=7)
                _, self.last_good_revision = \
                    self.nightly_runner.get_build_info(prev_date)
                self.bisect_inbound()
                return
            else:
                self._logger.info("(no more options with %s)" % self.appname)
                sys.exit()

        info = None
        while 1:
            self._logger.info("Running nightly for %s" % mid_date)
            if self.nightly_runner.start(mid_date):
                info = self.nightly_runner.get_app_info()
                self.found_repo = info['application_repository']
                if mid_date == bad_date:
                    self.print_range(good_date, bad_date)
                break
            else:
                if mid_date == bad_date:
                    sys.exit("Unable to get valid builds within the given"
                             " range. You should try to launch mozregression"
                             " again with a larger date range.")
            mid_date += datetime.timedelta(days=1)

        self.prev_date = self.curr_date
        self.curr_date = mid_date

        verdict = self._get_verdict('nightly')
        self.nightly_runner.stop()
        if verdict == 'g':
            self.last_good_revision = info['application_changeset']
            self.print_nightly_regression_progress(good_date, bad_date,
                                                   mid_date, bad_date)
            self.bisect_nightlies(mid_date, bad_date)
        elif verdict == 'b':
            self.first_bad_revision = info['application_changeset']
            self.print_nightly_regression_progress(good_date, bad_date,
                                                   good_date, mid_date)
            self.bisect_nightlies(good_date, mid_date)
        elif verdict == 's':
            # skip -- go 1 day further down
            self.bisect_nightlies(good_date, bad_date, skips=skips+1)
        elif verdict == 'e':
            self.nightly_runner.stop()
            good_date_string = '%04d-%02d-%02d' % (good_date.year,
                                                   good_date.month,
                                                   good_date.day)
            bad_date_string = '%04d-%02d-%02d' % (bad_date.year,
                                                  bad_date.month,
                                                  bad_date.day)
            self._logger.info('Newest known good nightly: %s'
                              % good_date_string)
            self._logger.info('Oldest known bad nightly: %s' % bad_date_string)
            self._logger.info('To resume, run:')
            self.nightly_runner.print_resume_info(good_date_string,
                                                  bad_date_string)
            return
        else:
            # retry -- since we're just calling ourselves with the same
            # parameters, it does the same thing again
            self.bisect_nightlies(good_date, bad_date)

    def get_pushlog_url(self, good_date, bad_date):
        # if we don't have precise revisions, we need to resort to just
        # using handwavey dates
        if not self.last_good_revision or not self.first_bad_revision:
            # pushlogs are typically done with the oldest date first
            if good_date < bad_date:
                start = good_date
                end = bad_date
            else:
                start = bad_date
                end = good_date
            return "%s/pushloghtml?startdate=%s&enddate=%s" % (self.found_repo,
                                                               start, end)

        return "%s/pushloghtml?fromchange=%s&tochange=%s" % (
            self.found_repo, self.last_good_revision, self.first_bad_revision)


def parse_args():
    usage = ("\n"
             " %(prog)s [OPTIONS]"
             " [[--bad BAD_DATE]|[--bad-release BAD_RELEASE]]"
             " [[--good GOOD_DATE]|[--good-release GOOD_RELEASE]]"
             "\n"
             " %(prog)s [OPTIONS]"
             " --inbound --bad-rev BAD_REV --good-rev GOOD_REV")

    parser = ArgumentParser(usage=usage)
    parser.add_argument("--version", action="version", version=__version__,
                        help="print the mozregression version number and exits")

    parser.add_argument("-b", "--bad",
                        metavar="YYYY-MM-DD",
                        dest="bad_date",
                        help="first known bad nightly build, default is today")

    parser.add_argument("-g", "--good",
                        metavar="YYYY-MM-DD",
                        dest="good_date",
                        help="last known good nightly build")

    parser.add_argument("--bad-release",
                        type=int,
                        help=("first known bad nightly build. This option is"
                              " incompatible with --bad."))

    parser.add_argument("--good-release",
                        type=int,
                        help=("last known good nightly build. This option is"
                              " incompatible with --good."))

    parser.add_argument("--inbound",
                        action="store_true",
                        help=("use inbound instead of nightlies (use --good-rev"
                              " and --bad-rev options"))

    parser.add_argument("--bad-rev", dest="first_bad_revision",
                        help="first known bad revision (use with --inbound)")

    parser.add_argument("--good-rev", dest="last_good_revision",
                        help="last known good revision (use with --inbound)")

    parser.add_argument("-e", "--addon",
                        dest="addons",
                        action='append',
                        default=[],
                        metavar="PATH1",
                        help="an addon to install; repeat for multiple addons")

    parser.add_argument("-p", "--profile",
                        metavar="PATH",
                        help="profile to use with nightlies")

    parser.add_argument("-a", "--arg",
                        dest="cmdargs",
                        action='append',
                        default=[],
                        metavar="ARG1",
                        help=("a command-line argument to pass to the"
                              " application; repeat for multiple arguments"))

    parser.add_argument("-n", "--app",
                        choices=('firefox', 'fennec', 'thunderbird', 'b2g'),
                        default="firefox",
                        help="application name. Default: %(default)s")

    parser.add_argument("--inbound-branch",
                        metavar="[tracemonkey|mozilla-1.9.2]",
                        help="inbound branch name on ftp.mozilla.org")

    parser.add_argument("--bits",
                        choices=("32", "64"),
                        default=mozinfo.bits,
                        help=("force 32 or 64 bit version (only applies to"
                              " x86_64 boxes). Default: %(default)s bits"))

    parser.add_argument("--persist",
                        help="the directory in which files are to persist")

    commandline.add_logging_group(parser)
    options = parser.parse_args()
    options.bits = parse_bits(options.bits)
    return options


def cli():
    default_bad_date = str(datetime.date.today())
    default_good_date = "2009-01-01"
    options = parse_args()
    logger = commandline.setup_logging("mozregression", options, {"mach": sys.stdout})

    inbound_runner = None
    if options.app in ("firefox", "fennec", "b2g") and not (mozinfo.os == 'win' and options.bits == 64):
        inbound_runner = InboundRunner(appname=options.app,
                                       addons=options.addons,
                                       inbound_branch=options.inbound_branch,
                                       profile=options.profile,
                                       cmdargs=options.cmdargs,
                                       bits=options.bits,
                                       persist=options.persist)

    if options.inbound:
        if not options.last_good_revision or not options.first_bad_revision:
            sys.exit("If bisecting inbound, both --good-rev and --bad-rev"
                     " must be set")
        bisector = Bisector(None, inbound_runner, appname=options.app,
                            last_good_revision=options.last_good_revision,
                            first_bad_revision=options.first_bad_revision)
        app = bisector.bisect_inbound
    else:
        if not options.bad_release and not options.bad_date:
            options.bad_date = default_bad_date
            logger.info("No 'bad' date specified, using %s" % options.bad_date)
        elif options.bad_release and options.bad_date:
            sys.exit("Options '--bad_release' and '--bad_date' are"
                     " incompatible.")
        elif options.bad_release:
            options.bad_date = date_of_release(options.bad_release)
            if options.bad_date is None:
                sys.exit("Unable to find a matching date for release "
                         + str(options.bad_release))
            logger.info("Using 'bad' date %s for release %s"
                        % (options.bad_date, options.bad_release))
        if not options.good_release and not options.good_date:
            options.good_date = default_good_date
            logger.info("No 'good' date specified, using %s"
                        % options.good_date)
        elif options.good_release and options.good_date:
            sys.exit("Options '--good_release' and '--good_date'"
                     " are incompatible.")
        elif options.good_release:
            options.good_date = date_of_release(options.good_release)
            if options.good_date is None:
                sys.exit("Unable to find a matching date for release "
                         + str(options.good_release))
            logger.info("Using 'good' date %s for release %s"
                        % (options.good_date, options.good_release))

        nightly_runner = NightlyRunner(appname=options.app, addons=options.addons,
                                       inbound_branch=options.inbound_branch,
                                       profile=options.profile,
                                       cmdargs=options.cmdargs,
                                       bits=options.bits,
                                       persist=options.persist)
        bisector = Bisector(nightly_runner, inbound_runner,
                            appname=options.app)
        app = lambda: bisector.bisect_nightlies(get_date(options.good_date),
                                                get_date(options.bad_date))
    try:
        app()
    except KeyboardInterrupt:
        sys.exit("\nInterrupted.")
    except errors.MozRegressionError as exc:
        sys.exit(str(exc))


if __name__ == "__main__":
    cli()




class MozRegressionError(Exception):
    """Base class for mozregression errors."""

class Win64NoAvailableBuildError(MozRegressionError):
    def __init__(self):
        MozRegressionError.__init__(self,
                                    "No builds available for 64 bit Windows"
                                    " (try specifying --bits=32)")

class WinTooOldBuildError(MozRegressionError):
    def __init__(self):
        MozRegressionError.__init__(self,
                                    "Can't run Windows builds before"
                                    " 2010-03-18")

class DateFormatError(MozRegressionError):
    def __init__(self, date_string):
        MozRegressionError.__init__(self,
                                    "Incorrect date format: `%s`" % date_string)

class DownloadError(MozRegressionError):
    pass



# first draft at something we may find useful later
def getPushRevisions(startRev, endRev, tree):
    if tree == 'mozilla-inbound':
        baseURL = 'https://hg.mozilla.org/integration/mozilla-inbound/'
    elif tree == 'mozilla-central':
        baseURL = 'https://hg.mozilla.org/mozilla-central/'
    else:
        raise Exception("Unknown tree '%s'" % tree)

    revisions = []
    r = requests.get('%sjson-pushes?fromchange=%s&tochange=%s'% (baseURL,
                                                                 startRev,
                                                                 endRev))
    pushlog = json.loads(r.content)
    for pushid in sorted(pushlog.keys()):
        push = pushlog[pushid]
        revisions.append((push['changesets'][-1], push['date']))




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import datetime
import os
import re
import sys
from BeautifulSoup import BeautifulSoup
import requests

from mozregression import errors

def format_date(date):
    return date.strftime('%Y-%m-%d')


def get_date(date_string):
    regex = re.compile(r'(\d{4})\-(\d{1,2})\-(\d{1,2})')
    matched = regex.match(date_string)
    if not matched:
        raise errors.DateFormatError(date_string)
    return datetime.date(int(matched.group(1)),
                         int(matched.group(2)),
                         int(matched.group(3)))


def update_download_progress(percent):
    sys.stdout.write("===== Downloaded %d%% =====\r" % percent)
    sys.stdout.flush()
    if percent >= 100:
        sys.stdout.write("\n")


def download_url(url, dest=None):
    chunk_size = 16 * 1024
    bytes_so_far = 0.0
    tmp_file = dest + ".part"
    request = requests.get(url, stream=True)
    total_size = int(request.headers['Content-length'].strip())
    if dest is None:
        dest = os.path.basename(url)

    with open(tmp_file, 'wb') as ftmp:
        # write the file to the tmp_file
        for chunk in request.iter_content(chunk_size=chunk_size):
            # Filter out Keep-Alive chunks.
            if not chunk:
                continue
            bytes_so_far += chunk_size
            ftmp.write(chunk)
            percent = (bytes_so_far / total_size) * 100
            update_download_progress(percent)
    # move the temp file to the dest
    os.rename(tmp_file, dest)

    return dest


def url_links(url, regex=None, auth=None):
    response = requests.get(url, auth=auth)
    response.raise_for_status()

    soup = BeautifulSoup(response.text)

    if regex:
        if isinstance(regex, basestring):
            regex = re.compile(regex)
        match = regex.match
    else:
        match = lambda t: True

    # do not return a generator but an array, so we can store it for later use
    return [link.get('href')
            for link in soup.findAll('a')
            if match(link.get('href'))]

def date_of_release(release):
    """Provide the date of a release.
    The date is a string formated as "yyyy-mm-dd",
    and the release an integer.
    """
    releases = {
        5: "2011-04-12",
        6: "2011-05-24",
        7: "2011-07-05",
        8: "2011-08-16",
        9: "2011-09-27",
        10: "2011-11-08",
        11: "2011-12-20",
        12: "2012-01-31",
        13: "2012-03-13",
        14: "2012-04-24",
        15: "2012-06-05",
        16: "2012-07-16",
        17: "2012-08-27",
        18: "2012-10-08",
        19: "2012-11-19",
        20: "2013-01-07",
        21: "2013-02-19",
        22: "2013-04-01",
        23: "2013-05-13",
        24: "2013-06-24",
        25: "2013-08-05",
        26: "2013-09-16",
        27: "2013-10-28",
        28: "2013-12-09",
        29: "2014-02-03",
        30: "2014-03-17",
        31: "2014-04-28",
        32: "2014-06-09",
        33: "2014-07-21",
        34: "2014-09-02",
    }
    if release in releases:
        return releases[release]
    return None



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import datetime
import os
import mozinstall
import re
import sys
import tempfile
import mozinfo
from mozfile import rmtree
from mozprofile import FirefoxProfile, ThunderbirdProfile, Profile
from mozrunner import Runner
from optparse import OptionParser
import mozversion
import requests
from mozlog.structured import get_default_logger

from mozregression import errors
from mozregression.utils import get_date, download_url, url_links

from mozdevice import ADBAndroid, ADBHost

class Nightly(object):

    name = None  # abstract base class
    _monthlinks = {}
    lastdest = None
    tempdir = None
    app_name = None
    profile_class = Profile
    build_base_repo_name = "firefox"

    @staticmethod
    def _get_os_regex_suffix(bits, with_ext=True):
        if mozinfo.os == "win":
            if bits == 64:
                suffix, ext = ".*win64-x86_64", ".zip"
            else:
                suffix, ext = ".*win32", ".zip"
        elif mozinfo.os == "linux":
            if bits == 64:
                suffix, ext = ".*linux-x86_64", ".tar.bz2"
            else:
                suffix, ext = ".*linux-i686", ".tar.bz2"
        elif mozinfo.os == "mac":
            suffix, ext = r".*mac.*", "\.dmg"

        if with_ext:
            return '%s%s' % (suffix, ext)
        else:
            return suffix

    @staticmethod
    def _get_build_regex(name, bits, with_ext=True):
        name_prefix = name if ".*%s" % name is not None else ''
        suffix = Nightly._get_os_regex_suffix(bits, with_ext)
        return "%s%s" % (name_prefix, suffix)

    def __init__(self, inbound_branch=None, bits=mozinfo.bits, persist=None):
        self.inbound_branch = inbound_branch
        self.bits = bits
        self.persist = persist
        self.build_regex = self._get_build_regex(self.name, bits) + "$"
        self.build_info_regex = \
            self._get_build_regex(self.name, bits, with_ext=False) + "\.txt$"
        self._logger = get_default_logger('Regression Runner')

    def get_inbound_branch(self, date):
        raise NotImplementedError

    # cleanup functions

    def remove_tempdir(self):
        if self.tempdir:
            rmtree(self.tempdir)
            self.tempdir = None

    def remove_lastdest(self):
        if self.lastdest:
            os.remove(self.lastdest)
            self.lastdest = None

    def cleanup(self):
        self.remove_tempdir()
        if not self.persist:
            self.remove_lastdest()

    __del__ = cleanup

    # installation functions

    def get_destination(self, url, date):
        inbound_branch = self.get_inbound_branch(date)
        dest = os.path.basename(url)
        if self.persist is not None:
            if hasattr(date, "strftime"):
                date_str = date.strftime("%Y-%m-%d")
            else:
                date_str = date  # Might be just a number with inbound
            dest = os.path.join(self.persist,
                                "%s--%s--%s" % (date_str, inbound_branch, dest))
        return dest

    def download(self, date=datetime.date.today(), dest=None):
        url = self.get_build_url(date)
        if url:
            if not dest:
                dest = self.get_destination(url, date)
            if not self.persist:
                self.remove_lastdest()

            if os.path.exists(dest):
                self._logger.info("Using local file: %s" % dest)
            else:
                self._logger.info("Downloading build from: %s" % url)
                download_url(url, dest)
            self.dest = self.lastdest = dest
            return True
        else:
            return False

    def install(self):
        if not self.name:
            raise NotImplementedError("Can't invoke abstract base class")
        self.remove_tempdir()
        self.tempdir = tempfile.mkdtemp()
        self.binary = mozinstall.get_binary(
            mozinstall.install(src=self.dest, dest=self.tempdir),
            self.name)
        return True

    def get_build_info(self, date):
        url = self._get_build_url(date, self.build_info_regex, 'builds info')
        if url is not None:
            self._logger.info("Getting %s" % url)
            response = requests.get(url)
            if response.status_code == 200:
                for line in response.text.splitlines():
                    if '/rev/' in line:
                        # returns [repository, changeset]
                        return line.split('/rev/')

    def get_build_url(self, datestamp):
        return self._get_build_url(datestamp, self.build_regex, 'builds')

    def _get_build_url(self, datestamp, regex, what):
        url = "http://ftp.mozilla.org/pub/mozilla.org/" + \
            self.build_base_repo_name + "/nightly/"
        year = str(datestamp.year)
        month = "%02d" % datestamp.month
        day = "%02d" % datestamp.day
        inbound_branch = self.get_inbound_branch(datestamp)
        url += year + "/" + month + "/"

        link_regex = '^' + year + '-' + month + '-' + day + '-' \
                     + r'[\d-]+' + inbound_branch + '/$'
        cachekey = year + '-' + month
        if cachekey in self._monthlinks:
            monthlinks = self._monthlinks[cachekey]
        else:
            monthlinks = url_links(url)
            self._monthlinks[cachekey] = monthlinks

        # first parse monthly list to get correct directory
        matches = []
        for dirlink in monthlinks:
            if re.match(link_regex, dirlink):
                # now parse the page for the correct build url
                for link in url_links(url + dirlink, regex=regex):
                    matches.append(url + dirlink + link)
        if not matches:
            self._logger.info("Tried to get %s from %s that match '%s'"
                              " but didn't find any."
                              % (what, url, self.build_regex))
            return None
        else:
            return sorted(matches)[-1] # the most recent build url

    # functions for invoking nightly

    def get_app_info(self):
        return mozversion.get_version(binary=self.binary)

    def start(self, profile, addons, cmdargs):
        if profile:
            profile = self.profile_class(profile=profile, addons=addons)
        elif len(addons):
            profile = self.profile_class(addons=addons)
        else:
            profile = self.profile_class()

        process_args = {'processOutputLine': [self._logger.debug]}
        self.runner = Runner(binary=self.binary,
                             cmdargs=cmdargs,
                             profile=profile,
                             process_args=process_args)
        self.runner.start()
        return True

    def stop(self):
        self.runner.stop()

    def wait(self):
        self.runner.wait()


class ThunderbirdNightly(Nightly):
    app_name = 'thunderbird'
    name = 'thunderbird'
    build_base_repo_name = 'thunderbird'
    profile_class = ThunderbirdProfile

    def get_inbound_branch(self, date):
        # sneaking this in here
        if mozinfo.os == "win" and date < datetime.date(2010, 03, 18):
            # no .zip package for Windows, can't use the installer
            raise errors.WinTooOldBuildError()

        if date < datetime.date(2008, 7, 26):
            return "trunk"
        elif date < datetime.date(2009, 1, 9):
            return "comm-central"
        elif date < datetime.date(2010, 8, 21):
            return "comm-central-trunk"
        else:
            return "comm-central"


class FirefoxNightly(Nightly):
    app_name = 'firefox'
    name = 'firefox'
    profile_class = FirefoxProfile

    def get_inbound_branch(self, date):
        if date < datetime.date(2008, 6, 17):
            return "trunk"
        else:
            return "mozilla-central"


class FennecNightly(Nightly):
    app_name = 'fennec'
    name = 'fennec'
    profile_class = FirefoxProfile
    build_regex = r'fennec-.*\.apk'
    build_info_regex = r'fennec-.*\.txt'
    binary = 'org.mozilla.fennec/.App'
    bits = None
    build_base_repo_name = "mobile"

    def get_device_status(self):
        self.adbhost = ADBHost()
        if self.adbhost.devices():
            return True
        if "y" == raw_input("WARNING: no device connected."
                            " Connect a device and try again.\n"
                            "Try again? (y or n): "):
            return self.get_device_status()
        raise Exception("Aborting!")

    def __init__(self, inbound_branch=None, bits=mozinfo.bits, persist=None):
        self.inbound_branch = inbound_branch
        self.persist = persist
        if self.get_device_status():
            self.adb = ADBAndroid()
            if "y" != raw_input("WARNING: bisecting nightly fennec builds will"
                                " clobber your existing nightly profile."
                                " Continue? (y or n)"):
                raise Exception("Aborting!")

    def get_inbound_branch(self, date):
        return "mozilla-central-android"

    def install(self):
        self.adb.uninstall_app("org.mozilla.fennec")
        self.adb.install_app(self.dest)
        return True

    def start(self, profile, addons, cmdargs):
        self.adb.launch_fennec("org.mozilla.fennec")
        return True

    def stop(self):
        self.adb.stop_application("org.mozilla.fennec")
        return True

    def get_app_info(self):
        return mozversion.get_version(binary=self.dest)


class B2GNightly(Nightly):
    app_name = 'b2g'
    name = 'b2g'
    profile_class = Profile
    build_base_repo_name = 'b2g'

    def get_inbound_branch(self, date):
        return "mozilla-central"


class NightlyRunner(object):

    apps = {'thunderbird': ThunderbirdNightly,
            'fennec': FennecNightly,
            'firefox': FirefoxNightly,
            'b2g': B2GNightly}

    def __init__(self, addons=None, appname="firefox", inbound_branch=None,
                 profile=None, cmdargs=(), bits=mozinfo.bits, persist=None):
        self.app = self.apps[appname](inbound_branch=inbound_branch, bits=bits,
                                      persist=persist)
        self.addons = addons
        self.profile = profile
        self.persist = persist
        self.cmdargs = list(cmdargs)
        self.inbound_branch = inbound_branch
        self._logger = get_default_logger('Regression Runner')

    def install(self, date=datetime.date.today()):
        if not self.app.download(date=date):
            self._logger.info("Could not find build from %s" % date)
            return False  # download failed
        self._logger.info("Installing nightly")
        return self.app.install()

    def start(self, date=datetime.date.today()):
        if not self.install(date):
            return False
        info = self.get_app_info()
        if info is not None:
            self._logger.info("Starting nightly (revision: %s)"
                              % info['application_changeset'])
        else:
            self._logger.info("Starting nightly")
        if not self.app.start(self.profile, self.addons, self.cmdargs):
            return False
        return True

    def stop(self):
        self.app.stop()

    def wait(self):
        self.app.wait()

    def cleanup(self):
        self.app.cleanup()

    def get_build_info(self, date=datetime.date.today()):
        result = self.app.get_build_info(date)
        if result is None:
            self._logger.info("Failed to retrieve build repository and revision"
                              " from the build dir. Let's try to install it to"
                              " get the required metadata...")
            self.install(date)
            info = self.get_app_info()
            result = (info['application_repository'],
                      info['application_changeset'])
        return result

    def get_app_info(self):
        return self.app.get_app_info()

    def get_resume_options(self):
        info = ""
        app = self.app.app_name
        inbound_branch = self.app.inbound_branch
        bits = self.app.bits
        if app is not None:
            info += ' --app=%s' % app
        if len(self.addons) > 0:
            info += ' --addons=%s' % ",".join(self.addons)
        if self.profile is not None:
            info += ' --profile=%s' % self.profile
        if inbound_branch is not None:
            info += ' --inbound-branch=%s' % inbound_branch
        if bits is not None:
            info += ' --bits=%s' % bits
        if self.persist is not None:
            info += ' --persist=%s' % self.persist
        return info

    def print_resume_info(self, good_date_string, bad_date_string):
        self._logger.info('mozregression --good=%s --bad=%s%s'
                          % (good_date_string,
                             bad_date_string,
                             self.get_resume_options()))


def parse_bits(option_bits):
    """returns the correctly typed bits"""
    if option_bits == "32":
        return 32
    else:
        # if 64 bits is passed on a 32 bit system, it won't be honored
        return mozinfo.bits


def cli(args=sys.argv[1:]):
    """moznightly command line entry point"""

    # parse command line options
    parser = OptionParser()
    parser.add_option("-d", "--date", dest="date", help="date of the nightly",
                      metavar="YYYY-MM-DD", default=str(datetime.date.today()))
    parser.add_option("-a", "--addons", dest="addons",
                      help="an addon to install; repeat for multiple addons",
                      metavar="PATH1", default=[], action="append")
    parser.add_option("-p", "--profile", dest="profile",
                      help="path to profile to user", metavar="PATH")
    parser.add_option("-n", "--app", dest="app", help="application name",
                      type="choice",
                      metavar="[%s]" % "|".join(NightlyRunner.apps.keys()),
                      choices=NightlyRunner.apps.keys(),
                      default="firefox")
    parser.add_option("--inbound-branch", dest="inbound_branch",
                      help="inbound branch name on ftp.mozilla.org",
                      metavar="[tracemonkey|mozilla-1.9.2]", default=None)
    parser.add_option("--bits", dest="bits",
                      help="force 32 or 64 bit version (only applies to"
                      " x86_64 boxes)",
                      choices=("32", "64"), default=mozinfo.bits)
    parser.add_option("--persist", dest="persist",
                      help="the directory in which files are to persist ie."
                      " /Users/someuser/Documents")
    options, args = parser.parse_args(args)

    options.bits = parse_bits(options.bits)

    # run nightly
    runner = NightlyRunner(appname=options.app, addons=options.addons,
                           profile=options.profile,
                           inbound_branch=options.inbound_branch,
                           bits=options.bits, persist=options.persist)
    runner.start(get_date(options.date))
    try:
        runner.wait()
    except KeyboardInterrupt:
        runner.stop()


if __name__ == "__main__":
    cli()



import mozinfo
import sys
from optparse import OptionParser
import requests
import copy
from mozlog.structured import get_default_logger

from mozregression import errors
from mozregression.utils import url_links
from concurrent import futures

def get_repo_url(path='integration', inbound_branch='mozilla-inbound'):
    return "https://hg.mozilla.org/%s/%s" % (path, inbound_branch)

class PushLogsFinder(object):
    """
    Find pushlog json objects within two revisions.
    """
    def __init__(self, start_rev, end_rev, path='integration',
                 inbound_branch='mozilla-inbound'):
        self.start_rev = start_rev
        self.end_rev = end_rev
        self.path = path
        self.inbound_branch = inbound_branch

    def pushlog_url(self):
        return '%s/json-pushes?fromchange=%s&tochange=%s' % (
            get_repo_url(self.path, self.inbound_branch),
            self.start_rev, self.end_rev)

    def get_pushlogs(self):
        """
        Returns pushlog json objects (python dicts) sorted by date.
        """
        response = requests.get(self.pushlog_url())
        response.raise_for_status()
        # sort pushlogs by date
        return sorted(response.json().itervalues(),
                      key=lambda push: push['date'])


class InboundBuildData(object):
    """
    Represents the inbound build data avalaible, designed to be used in
    bisection.

    It allow the lazy loading of build data, by fetching only lower bound,
    higher bound and middle points when required.

    As some build folders are not valid - and we can know that only
    after really fetching data - there is some imprecision on data
    size which may reduce when you call ensure_limits or mid_point.

    An instance of InboundBuildData acts like a list, ensuring following
    methods:
     - len(data) # size
     - data[1:]  # splice
     - data[0]   # get data
    """
    def __init__(self, cache, raw_revisions, half_window_range=4):
        self._cache = cache
        self.raw_revisions = raw_revisions
        self.half_window_range = half_window_range
        self._logger = get_default_logger('Inbound Build Finder')

    def __len__(self):
        return len(self._cache)

    def __getslice__(self, smin, smax):
        return InboundBuildData(self._cache[smin:smax],
                               self.raw_revisions,
                               half_window_range=self.half_window_range)

    def __getitem__(self, i):
        return self._cache[i][0]

    def ensure_limits(self):
        """
        Ensure we know the min and max data, fetching it if necessary.
        """
        done = False
        while not done:
            size = len(self)
            if size == 0:
                # no limits to ensure
                return

            # first we detect if we need to fetch data, and which build id's
            range_min = []
            range_max = []

            # we first get an artifically large range (double) to optimize
            # the number of build data which will be fetched.
            #
            # this way, if we have a self.half_window_range of 4 (the default)
            # and for example we don't need the lowest bound, the number of
            # data to fetch for higher bound will be 8 (full range) instead
            # of 4.
            if self[0] is None:
                self._logger.debug("We need to fetch the inbound lower limit")
                bound = min(size, self.half_window_range*2)
                range_min.extend(range(0, bound))
            if self[-1] is None:
                self._logger.debug("We need to fetch the inbound higher limit")
                bound = max(0, size - self.half_window_range*2)
                range_max.extend(range(bound, size))

            # restrict the number of builds to fetch
            range_to_update = set(range_min + range_max)
            while len(range_to_update) > self.half_window_range*2:
                if len(range_min) > len(range_max):
                    range_min = range_min[:-1]
                else:
                    range_max = range_max[1:]
                range_to_update = set(range_min + range_max)

            if range_to_update:
                self._fetch(range_to_update)
                # we asked for some data, but it may be not suficient -
                # data may be invalid - let's check it by looping
            else:
                # data is fetched on min and max bounds, we can exit
                done = True

    def mid_point(self):
        """
        Return the middle index of the data. If 0, there is no middle
        point anymore.
        """
        while True:
            self.ensure_limits()

            size = len(self)
            mid = size / 2
            if mid == 0:
                return 0

            if self[mid] is None:
                self._logger.debug('We need to fetch the inbound mid point %d'
                                   % mid)
                # we need to fetch the middle of the data
                rmax = min(size, mid + self.half_window_range)
                rmin = max(0, mid - self.half_window_range)
                self._fetch(range(rmin, rmax))
                # maybe all fetched data was invalid and we need to
                # check it by looping
            else:
                return mid

    def _fetch(self, ids, max_retry=3):
        """
        Internal method to fetch some data.
        """
        # filter the already downloaded data
        builds_to_get = [i for i in ids if self[i] is None]
        size = len(self)
        self._logger.debug("We got %d inbound folders, we need to fetch %s"
                           % (size, sorted(builds_to_get)))

        nb_try = 0
        while builds_to_get:
            nb_try += 1
            with futures.ThreadPoolExecutor(max_workers=8) as executor:
                futures_results = {}
                for i in builds_to_get:
                    build_url, timestamp = self._cache[i][1], self._cache[i][2]
                    future = executor.submit(self._get_valid_build,
                                             build_url,
                                             timestamp,
                                             self.raw_revisions)
                    futures_results[future] = i
                for future in futures.as_completed(futures_results):
                    i = futures_results[future]
                    exc = future.exception()
                    if exc is not None:
                        must_raise = True
                        if isinstance(exc, requests.HTTPError):
                            if nb_try < max_retry:
                                self._logger.warning("Got HTTPError - retrying")
                                must_raise = False
                        if must_raise:
                            raise errors.DownloadError(
                                "Retrieving valid builds from %r generated an"
                                " exception: %s" % (self._cache[i][1],
                                                    exc))
                    else:
                        builds_to_get.remove(i)
                        self._cache[i][0] = future.result()
        # filter the builds that were invalids
        self._cache = [c for c in self._cache if c[0] is not False]
        self._logger.debug("Now we got %d inbound folders - %d were bad"
                           % (len(self), size - len(self)))

    def _get_valid_build(self, build_url, timestamp, raw_revisions):
        for link in url_links(build_url, regex=r'^.+\.txt$'):
            url = "%s/%s" % (build_url, link)
            response = requests.get(url)
            remote_revision = None
            for line in response.iter_lines():
                # Filter out Keep-Alive new lines.
                if not line:
                    continue
                parts = line.split('/rev/')
                if len(parts) == 2:
                    remote_revision = parts[1]
                    break  # for line
            if remote_revision:
                for revision in raw_revisions:
                    if remote_revision in revision:
                        return {
                            'revision': revision[:8],
                            'timestamp': timestamp,
                        }
        return False


class BuildsFinder(object):
    """
    Find builds information for builds within two revisions.
    """
    def __init__(self, bits=mozinfo.bits, os=mozinfo.os,
                 inbound_branch=None):
        self.bits = bits
        self.os = os
        self.inbound_branch = inbound_branch or self.default_inbound_branch
        self.build_base_url = self._get_build_base_url(self.inbound_branch)

    def _create_pushlog_finder(self, start_rev, end_rev):
        return PushLogsFinder(start_rev, end_rev,
                              inbound_branch=self.inbound_branch)

    def _get_build_base_url(self, inbound_branch):
        raise NotImplementedError()

    def _extract_paths(self):
        paths = filter(lambda l: l.isdigit(),
                       map(lambda l: l.strip('/'),
                           url_links(self.build_base_url)))
        return [(p, int(p)) for p in paths]

    def get_build_infos(self, start_rev, end_rev, range=60*60*4):
        """
        Returns build information for all builds between start_rev and end_rev
        """
        pushlogs_finder = self._create_pushlog_finder(start_rev, end_rev)

        pushlogs = pushlogs_finder.get_pushlogs()

        if not pushlogs:
            return []

        start_time = pushlogs[0]['date']
        end_time = pushlogs[-1]['date']
        
        build_urls = [("%s%s/" % (self.build_base_url, path), timestamp)
                      for path, timestamp in self._extract_paths()]

        build_urls_in_range = filter(lambda (u, t): t > (start_time - range)
                                     and t < (end_time + range), build_urls)
        # build empty cache
        cache = []
        for url, timestamp in sorted(build_urls_in_range, key=lambda b: b[1]):
            cache.append([None, url, timestamp])

        raw_revisions = [push['changesets'][-1] for push in pushlogs]
        return InboundBuildData(cache, raw_revisions)


class FennecBuildsFinder(BuildsFinder):
    default_inbound_branch = 'mozilla-inbound'

    def _get_build_base_url(self, inbound_branch):
        return "http://inbound-archive.pub.build.mozilla.org/pub/mozilla.org" \
            "/mobile/tinderbox-builds/%s-android/" % inbound_branch


class FirefoxBuildsFinder(BuildsFinder):
    build_base_os_part = {
        'linux': {32: 'linux', 64: 'linux64'},
        'win': {32: 'win32'},
        'mac': {64: 'macosx64'}
    }
    root_build_base_url = 'http://inbound-archive.pub.build.mozilla.org/pub' \
            '/mozilla.org/firefox/tinderbox-builds/%s-%s/'
    default_inbound_branch = 'mozilla-inbound'

    def _get_build_base_url(self, inbound_branch):
        if self.os == "win" and self.bits == 64:
            raise errors.Win64NoAvailableBuildError()
        return self.root_build_base_url % \
                (inbound_branch, self.build_base_os_part[self.os][self.bits])


class B2GBuildsFinder(FirefoxBuildsFinder):
    build_base_os_part = copy.deepcopy(FirefoxBuildsFinder.build_base_os_part)
    build_base_os_part['linux'][32] = 'linux32'
    root_build_base_url = 'http://ftp.mozilla.org/pub/mozilla.org/b2g'\
            '/tinderbox-builds/%s-%s_gecko/'
    default_inbound_branch = 'b2g-inbound'


def cli(args=sys.argv[1:]):

    parser = OptionParser()
    parser.add_option("--start-rev", dest="start_rev", help="start revision")
    parser.add_option("--end-rev", dest="end_rev", help="end revision")
    parser.add_option("--os", dest="os", help="override operating system "
                      "autodetection (mac, linux, win)", default=mozinfo.os)
    parser.add_option("--bits", dest="bits", help="override operating system "
                      "bits autodetection", default=mozinfo.bits)
    parser.add_option("-n", "--app", dest="app", help="application name "
                      "(firefox, fennec or b2g)",
                      metavar="[firefox|fennec|b2g]",
                      default="firefox")
    parser.add_option("--inbound-branch", dest="inbound_branch",
                      help="inbound branch name on ftp.mozilla.org",
                      metavar="[tracemonkey|mozilla-1.9.2]", default=None)

    options, args = parser.parse_args(args)
    if not options.start_rev or not options.end_rev:
        sys.exit("start revision and end revision must be specified")

    build_finders = {
        'firefox': FirefoxBuildsFinder,
        'b2g': B2GBuildsFinder,
        'fennec': FennecBuildsFinder
    }
    
    if options.inbound_branch:
        inbound_branch = options.inbound_branch
    else:
        inbound_branch = build_finder.default_inbound_branch

    build_finder = build_finders[options.app](os=options.os, bits=options.bits,
                                              inbound_branch=inbound_branch)
    
    revisions = build_finder.get_build_infos(options.start_rev,
                                             options.end_rev,
                                             range=60*60*12)
    print("Revision, Timestamp")
    for revision in revisions:
        print("%s %s" % (revision['revision'], revision['timestamp']))

if __name__ == "__main__":
    cli()



import sys
import mozinfo
from optparse import OptionParser
from mozlog.structured import get_default_logger

from mozregression.runnightly import FennecNightly, FirefoxNightly, \
    B2GNightly, NightlyRunner, parse_bits
from mozregression.inboundfinder import FirefoxBuildsFinder, \
    FennecBuildsFinder, B2GBuildsFinder
from mozregression.utils import url_links, get_date


class FirefoxInbound(FirefoxNightly):

    repo_name = None

    def __init__(self, bits=mozinfo.bits, persist=None, inbound_branch=None):
        self.persist = persist
        self.build_regex = self._get_build_regex(self.name, bits)
        self.bits = bits
        self.build_finder = FirefoxBuildsFinder(bits=bits, inbound_branch=inbound_branch)
        self.inbound_branch = inbound_branch or FirefoxBuildsFinder.default_inbound_branch
        self._logger = get_default_logger('Regression Runner')

    def get_build_url(self, timestamp):
        base_url = "%s%s/" % (self.build_finder.build_base_url, timestamp)
        matches = [base_url + url
                   for url in url_links(base_url, regex=self.build_regex)]
        matches.sort()
        return matches[-1]  # the most recent build url

    def get_inbound_branch(self, date):
        return "mozilla-inbound"


class FennecInbound(FennecNightly):

    repo_name = None

    def __init__(self, persist=None, inbound_branch=None):
        self.persist = persist
        self.inbound_branch = inbound_branch or FennecBuildsFinder.default_inbound_branch
        self.build_finder = FennecBuildsFinder(inbound_branch=inbound_branch)
        self._logger = get_default_logger('Regression Runner')

    def get_build_url(self, timestamp):
        base_url = "%s%s/" % (self.build_finder.build_base_url, timestamp)
        matches = [base_url + url
                   for url in url_links(base_url, regex=self.build_regex)]
        matches.sort()
        return matches[-1]  # the most recent build url

    def get_inbound_branch(self, date):
        return "mozilla-inbound"


class B2GInbound(B2GNightly):

    repo_name = None

    def __init__(self, **kwargs):
        B2GNightly.__init__(self, **kwargs)
        self.inbound_branch = (kwargs['inbound_branch'] or
                               B2GBuildsFinder.default_inbound_branch)
        self.build_finder = B2GBuildsFinder(bits=self.bits,
                                            inbound_branch=self.inbound_branch)

    def get_build_url(self, timestamp):
        base_url = "%s%s/" % (self.build_finder.build_base_url, timestamp)
        matches = [base_url + url
                   for url in url_links(base_url, regex=self.build_regex)]
        matches.sort()
        return matches[-1]  # the most recent build url

    def get_inbound_branch(self, date):
        return "b2g-inbound"


class InboundRunner(NightlyRunner):

    def __init__(self, addons=None, appname="firefox", repo_name=None,
                 profile=None, cmdargs=[], bits=mozinfo.bits, persist=None,
                 inbound_branch=None):
        if appname == 'firefox':
            self.app = FirefoxInbound(bits=bits, persist=persist,
                                      inbound_branch=inbound_branch)
        elif appname == 'b2g':
            self.app = B2GInbound(bits=bits, persist=persist,
                                  inbound_branch=inbound_branch)
        else:
            self.app = FennecInbound(persist=persist,
                                     inbound_branch=inbound_branch)
        self.app_name = appname
        self.bits = bits
        self.addons = addons
        self.profile = profile
        self.persist = persist
        self.inbound_branch = self.app.inbound_branch
        self.cmdargs = list(cmdargs)
        self._logger = get_default_logger('Regression Runner')

    def print_resume_info(self, last_good_revision, first_bad_revision):
        self._logger.info('mozregression --good-rev=%s --bad-rev=%s%s'
                          % (last_good_revision,
                             first_bad_revision,
                             self.get_resume_options()))


def cli(args=sys.argv[1:]):
    parser = OptionParser()
    parser.add_option("--timestamp", dest="timestamp", help="timestamp of "
                      "inbound build")
    parser.add_option("-a", "--addon", dest="addons",
                      help="an addon to install; repeat for multiple addons",
                      metavar="PATH1", default=[], action="append")
    parser.add_option("-p", "--profile", dest="profile",
                      help="path to profile to user", metavar="PATH")
    parser.add_option("--bits", dest="bits",
                      help="force 32 or 64 bit version (only applies to"
                      " x86_64 boxes)",
                      choices=("32", "64"), default=mozinfo.bits)
    parser.add_option("--persist", dest="persist",
                      help="the directory in which files are to persist"
                      " ie. /Users/someuser/Documents")
    parser.add_option("--inbound-branch", dest="inbound_branch",
                      help="inbound branch name on ftp.mozilla.org",
                      metavar="[tracemonkey|mozilla-1.9.2]", default=None)

    options, args = parser.parse_args(args)
    if not options.timestamp:
        sys.exit("timestamp must be specified")
    options.bits = parse_bits(options.bits)
    runner = InboundRunner(addons=options.addons, profile=options.profile,
                           bits=options.bits, persist=options.persist,
                           inbound_branch=options.inbound_branch)
    runner.start(get_date(options.date))
    try:
        runner.wait()
    except KeyboardInterrupt:
        runner.stop()

if __name__ == "__main__":
    cli()



import os
import sys
from setuptools import setup
from mozregression import __version__

if sys.version_info < (2, 7) or sys.version_info > (3, 0):
    sys.exit("mozregression currently require python >=2.7 and <3.")

desc = """Regression range finder for Mozilla nightly builds"""
long_desc = """Interactive regression range finder for Mozilla nightly builds.

For more information see the mozregression website: http://mozilla.github.io/mozregression/"""

setup(name="mozregression",
      version=__version__,
      description=desc,
      long_description=long_desc,
      author='Mozilla Automation and Tools Team',
      author_email='tools@lists.mozilla.org',
      url='http://github.com/mozilla/mozregression',
      license='MPL 1.1/GPL 2.0/LGPL 2.1',
      packages=['mozregression'],
      entry_points="""
          [console_scripts]
          mozregression = mozregression.regression:cli
          moznightly = mozregression.runnightly:cli
        """,
      platforms =['Any'],
      install_requires = ['httplib2 >= 0.6.0',
                          'mozcommitbuilder >= 0.4.10',
                          'mozfile >= 0.1',
                          'mozprofile >= 0.4',
                          'mozrunner >= 5.14',
                          'BeautifulSoup >= 3.0.4',
                          'mozinstall >= 1.4',
                          'mozinfo >= 0.4',
                          'mozlog >= 2.7',
                          'mozversion >= 0.5',
                          'requests >= 2.1',
                          'futures >= 2.1.6',
                          'mozdevice >= 0.43'
                          ],
      tests_require=['mock'],
      test_suite='tests',
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Console',
                   'Intended Audience :: Developers',
                   'Operating System :: OS Independent'
                  ]
     )
